﻿/****************************************************************************/
/* Copyright (C) 2009-2011, Heartland.Data.co.  All Rights Reserved.		*/
/*																			*/
/* Title	: DT10 Driver	[All]											*/
/* Author	: HLDC															*/
/****************************************************************************/

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>

#include "dt10_tpdrv.h"

#define DEVNAME "/dev/dt10drvasync"	/* Device file  */

/* ========== Parameter ========== */
#define		DT_UINT32				unsigned int	/* Test Point argument type */
#define		DT_VARIABLE_BIT			0x02			/* Memory Dump MASK */
#define		DT_EVTTRG_BIT			0x08			/* EventTrigger */
/* ============================================ */


static int fd = -1;

/*===============================================================================
	Func	: _TP_BusOutOpen
	Desc	: Open device driver for Test Point.
===============================================================================*/
void	_TP_BusOutOpen(void)
{
	fd = open(DEVNAME, O_RDWR);
	if( fd < 0 )
	{
		perror("open");
	}
}

/*===============================================================================
	Func	: _TP_BusOutClose
	Desc	: Close device driver for Test Point.
===============================================================================*/
void	_TP_BusOutClose(void)
{
	if( fd >= 0 )
	{
		close(fd);
		fd = -1;
	}
}

/*===============================================================================
	Func	: _TP_BusOut
	Desc	: Called by Test Point
===============================================================================*/
void _TP_BusOut( DT_UINT32 addr, DT_UINT32 dat )
{
	int	ret;
	dt10_tp	tp;	

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr[0]	= addr;
	tp.data[0]	= dat;
	tp.count	= 1;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}

/*===============================================================================
	Func	: _TP_MemoryOutput
	Desc	: Called by Variable Test Point
===============================================================================*/
void _TP_MemoryOutput( DT_UINT32 addr, DT_UINT32 dat, void *value, DT_UINT32 size)
{
	dt10_tp	tp;	
	int	i, cnt = 0;
	int	ret;
	unsigned char *p;

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr[0]	= addr | DT_VARIABLE_BIT;			/* Memory Dump Start */ 
	tp.data[0]	= dat;
	cnt += 1;

	if (size >= MAX_TP_DATA) size = MAX_TP_DATA;
	if (value) {
		p = (unsigned char *)value;
		for (i = 0; i < size; i += 2) {
			tp.data[cnt] = p[i];
			if ((i + 1) < size) {
				tp.data[cnt] |= p[i+1] << 8;
			}
			tp.addr[cnt] = DT_VARIABLE_BIT;
			if (i == 0) {
				tp.addr[cnt] |= size << 8;
			}
			cnt++;
		}
	}
	tp.count = cnt;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}


/*===============================================================================
	Func	: _TP_EventTrigger
	Desc	: Called by Event Trigger Test Point
===============================================================================*/
void _TP_EventTrigger( DT_UINT32 addr, DT_UINT32 dat)
{
	dt10_tp	tp;	
	int	ret;

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr[0]	= addr | DT_EVTTRG_BIT ;
	tp.data[0]	= ((unsigned short)(dat & 0x0FFF)) << 4 ;
	tp.count	= 1;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}
