﻿/****************************************************************************/
/* Copyright (C) 2009-2011, Heartland.Data.co.  All Rights Reserved.		*/
/*																			*/
/* Title	: DT10 Driver	[All]											*/
/* Author	: HLDC															*/
/****************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/string.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/miscdevice.h>

#include "dt10_tpdrv.h"

#define LICENSE "GPL v2"

static void *cookie = NULL;
#define	FLASH_ADDR	0xF0000000   /* Please set beginning address in the virtual memory space. */
#define	FLASH_SIZE	0x02000000	/* Please set the Flash size */

/*****************Called by Test Point in the kernel space*****************/
static void async_write(dt10_tp tp)
{
	unsigned char *p;
	int	i;

	if (cookie == NULL) {
		cookie = ioremap(FLASH_ADDR, FLASH_SIZE);
	}

	for( i = 0; i < tp.count; i++ )
	{
		p = (unsigned char *)cookie;
		p += tp.addr[i];
		outw( tp.data[i], (unsigned int)p );
	}
}

void _TP_BusOut( unsigned int addr, unsigned int dat )
{
	dt10_tp	tp;	

	tp.addr[0]	= addr;
	tp.data[0]	= dat;
	tp.count	= 1;
	async_write(tp);
}
EXPORT_SYMBOL(_TP_BusOut);

void _TP_MemoryOutput( unsigned int addr, unsigned int dat, void *value, unsigned int size)
{
	dt10_tp	tp;	
	int	i, cnt = 0;
	unsigned char *p;

	tp.addr[0]	= addr | 0x02;			/* Memory Dump Start */ 
	tp.data[0]	= dat;
	cnt += 1;

	if (size >= MAX_TP_DATA) size = MAX_TP_DATA;
	if (value) {
		p = (unsigned char *)value;
		for (i = 0; i < size; i += 2) {
			tp.data[cnt] = p[i];
			if ((i + 1) < size) {
				tp.data[cnt] |= p[i+1] << 8;
			}
			tp.addr[cnt] = 0x02;
			if (i == 0) {
				tp.addr[cnt] |= size << 8;
			}
			cnt++;
		}
	}
	tp.count = cnt;
	async_write(tp);
}
EXPORT_SYMBOL(_TP_MemoryOutput);

void _TP_EventTrigger( unsigned int addr, unsigned int dat)
{
	dt10_tp	tp;

	tp.addr[0]	= addr | 0x08;
	tp.data[0]	= ((unsigned short)(dat & 0x0FFF)) << 4;
	tp.count	= 1;
	async_write(tp);
}
EXPORT_SYMBOL(_TP_EventTrigger);

/********************************************************************************/

/*===============================================================================
	Func	: dt10drvasync_open
	Desc	: This function is executed when opening the device driver.
===============================================================================*/
static int dt10drvasync_open(struct inode *inode, struct file *filp)
{
	if (cookie == NULL) {
		cookie = ioremap(FLASH_ADDR, FLASH_SIZE);
	}
	pr_debug("dt10drvasync: open\n");
	return 0;
}

/*===============================================================================
    Func	: dt10drvasync_write
    Desc	: This function is executed upon write instruction.
===============================================================================*/
static int dt10drvasync_write(struct file *filp, char *buff, size_t count, loff_t *pos)
{

	dt10_tp *tp;
	unsigned char *p;
	int	i;

	tp = (dt10_tp *)buff ;
	for( i = 0; i < tp->count; i++ )
	{
		p = (unsigned char *)cookie ;
		p += tp->addr[i];
		outw( tp->data[i], (unsigned int)p );
	}
	return count ;

}

/*===============================================================================
	Func	: dt10drvasync_release
	Desc	: This function is executed when closing the device driver.
===============================================================================*/
static int dt10drvasync_release(struct inode *inode, struct file *filp)
{
	iounmap( cookie );
	pr_debug("dt10drvasync: release\n");
	return 0;
}

/*===============================================================================
	Func	: file_operations driver_fops
===============================================================================*/
static struct file_operations driver_fops = {
	.write = dt10drvasync_write,
	.open = dt10drvasync_open,
	.release = dt10drvasync_release,
};

/*===============================================================================
	Func	: miscdevice misc
===============================================================================*/
static struct miscdevice misc = {
	.minor = MISC_DYNAMIC_MINOR,
	.name = "dt10drvasync",
	.fops = &driver_fops,
};

/*===============================================================================
	Func	: init_module_async
	Desc	: This function is executed when installing the device driver.
===============================================================================*/
int __init init_module_async(void)
{
	int ret;
	
	ret = misc_register(&misc);
	if (ret < 0) {
		printk(KERN_ERR "dt10drvasync: misc_register() failed\n");
	}
	printk("dt10drvasync: Success init_module.\n");

	return 0;
}

/*===============================================================================
	Func	: cleanup_module_async
	Desc	: This function is executed when uninstalling the device driver.
===============================================================================*/
void __exit cleanup_module_async(void)
{
	misc_deregister(&misc);
	pr_debug("dt10drvasync: cleanup_module\n");
}

module_init(init_module_async);
module_exit(cleanup_module_async);

MODULE_LICENSE(LICENSE);

